@@ -20,6 +20,10 @@
 
 #include "CipherKey.h"
 
+namespace encfs {
+
 AbstractCipherKey::AbstractCipherKey() {}
 
 AbstractCipherKey::~AbstractCipherKey() {}
+
+}  // namespace encfs
