@@ -18,38 +18,34 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "encfs.h"
-
-#include "config.h"
-
-#include <openssl/blowfish.h>
-#include <openssl/sha.h>
-#include <openssl/rand.h>
-#include <openssl/err.h>
-#include <openssl/hmac.h>
-
-#include "SSL_Cipher.h"
-#include "Range.h"
-#include "MemoryPool.h"
-#include "Mutex.h"
-
+#include "internal/easylogging++.h"
 #include <cstring>
-#include <ctime>
-
+#include <openssl/crypto.h>
+#include <openssl/err.h>
+#include <openssl/evp.h>
+#include <openssl/hmac.h>
+#include <openssl/ossl_typ.h>
+#include <openssl/rand.h>
+#include <pthread.h>
+#include <string>
 #include <sys/mman.h>
 #include <sys/time.h>
 
-#include <rlog/rlog.h>
-#include <rlog/Error.h>
-
-#include "i18n.h"
+#include "Cipher.h"
+#include "Error.h"
+#include "Interface.h"
+#include "Mutex.h"
+#include "Range.h"
+#include "SSL_Compat.h"
+#include "SSL_Cipher.h"
+#include "intl/gettext.h"
 
 using namespace std;
-using namespace rel;
-using namespace rlog;
+
+namespace encfs {
 
 const int MAX_KEYLENGTH = 32;  // in bytes (256 bit)
-const int MAX_IVLENGTH = 16; // 128 bit (AES block size, Blowfish has 64)
+const int MAX_IVLENGTH = 16;   // 128 bit (AES block size, Blowfish has 64)
 const int KEY_CHECKSUM_BYTES = 4;
 
 #ifndef MIN
@@ -79,19 +75,19 @@ int BytesToKey(int keyLen, int ivLen, const EVP_MD *md,
   int nkey = key ? keyLen : 0;
   int niv = iv ? ivLen : 0;
 
-  EVP_MD_CTX cx;
-  EVP_MD_CTX_init(&cx);
+  EVP_MD_CTX *cx = EVP_MD_CTX_new();
+  EVP_MD_CTX_init(cx);
 
   for (;;) {
-    EVP_DigestInit_ex(&cx, md, NULL);
-    if (addmd++) EVP_DigestUpdate(&cx, mdBuf, mds);
-    EVP_DigestUpdate(&cx, data, dataLen);
-    EVP_DigestFinal_ex(&cx, mdBuf, &mds);
+    EVP_DigestInit_ex(cx, md, NULL);
+    if (addmd++) EVP_DigestUpdate(cx, mdBuf, mds);
+    EVP_DigestUpdate(cx, data, dataLen);
+    EVP_DigestFinal_ex(cx, mdBuf, &mds);
 
     for (unsigned int i = 1; i < rounds; ++i) {
-      EVP_DigestInit_ex(&cx, md, NULL);
-      EVP_DigestUpdate(&cx, mdBuf, mds);
-      EVP_DigestFinal_ex(&cx, mdBuf, &mds);
+      EVP_DigestInit_ex(cx, md, NULL);
+      EVP_DigestUpdate(cx, mdBuf, mds);
+      EVP_DigestFinal_ex(cx, mdBuf, &mds);
     }
 
     int offset = 0;
@@ -111,7 +107,7 @@ int BytesToKey(int keyLen, int ivLen, const EVP_MD *md,
     }
     if ((nkey == 0) && (niv == 0)) break;
   }
-  EVP_MD_CTX_cleanup(&cx);
+  EVP_MD_CTX_free(cx);
   OPENSSL_cleanse(mdBuf, sizeof(mdBuf));
 
   return keyLen;
@@ -158,12 +154,12 @@ int TimedPBKDF2(const char *pass, int passlen, const unsigned char *salt,
 static Interface BlowfishInterface("ssl/blowfish", 3, 0, 2);
 static Interface AESInterface("ssl/aes", 3, 0, 2);
 
-#if defined(HAVE_EVP_BF)
+#ifndef OPENSSL_NO_BF
 
 static Range BFKeyRange(128, 256, 32);
 static Range BFBlockRange(64, 4096, 8);
 
-static shared_ptr<Cipher> NewBFCipher(const Interface &iface, int keyLen) {
+static std::shared_ptr<Cipher> NewBFCipher(const Interface &iface, int keyLen) {
   if (keyLen <= 0) keyLen = 160;
 
   keyLen = BFKeyRange.closest(keyLen);
@@ -171,7 +167,7 @@ static shared_ptr<Cipher> NewBFCipher(const Interface &iface, int keyLen) {
   const EVP_CIPHER *blockCipher = EVP_bf_cbc();
   const EVP_CIPHER *streamCipher = EVP_bf_cfb();
 
-  return shared_ptr<Cipher>(new SSL_Cipher(
+  return std::shared_ptr<Cipher>(new SSL_Cipher(
       iface, BlowfishInterface, blockCipher, streamCipher, keyLen / 8));
 }
 
@@ -182,12 +178,13 @@ static bool BF_Cipher_registered =
                      BFKeyRange, BFBlockRange, NewBFCipher);
 #endif
 
-#if defined(HAVE_EVP_AES)
+#ifndef OPENSSL_NO_AES
 
 static Range AESKeyRange(128, 256, 64);
 static Range AESBlockRange(64, 4096, 16);
 
-static shared_ptr<Cipher> NewAESCipher(const Interface &iface, int keyLen) {
+static std::shared_ptr<Cipher> NewAESCipher(const Interface &iface,
+                                            int keyLen) {
   if (keyLen <= 0) keyLen = 192;
 
   keyLen = AESKeyRange.closest(keyLen);
@@ -213,8 +210,8 @@ static shared_ptr<Cipher> NewAESCipher(const Interface &iface, int keyLen) {
       break;
   }
 
-  return shared_ptr<Cipher>(new SSL_Cipher(iface, AESInterface, blockCipher,
-                                           streamCipher, keyLen / 8));
+  return std::shared_ptr<Cipher>(new SSL_Cipher(
+      iface, AESInterface, blockCipher, streamCipher, keyLen / 8));
 }
 
 static bool AES_Cipher_registered =
@@ -233,12 +230,12 @@ class SSLKey : public AbstractCipherKey {
   // followed by iv of _ivLength bytes,
   unsigned char *buffer;
 
-  EVP_CIPHER_CTX block_enc;
-  EVP_CIPHER_CTX block_dec;
-  EVP_CIPHER_CTX stream_enc;
-  EVP_CIPHER_CTX stream_dec;
+  EVP_CIPHER_CTX *block_enc;
+  EVP_CIPHER_CTX *block_dec;
+  EVP_CIPHER_CTX *stream_enc;
+  EVP_CIPHER_CTX *stream_dec;
 
-  HMAC_CTX mac_ctx;
+  HMAC_CTX *mac_ctx;
 
   SSLKey(int keySize, int ivLength);
   ~SSLKey();
@@ -254,6 +251,17 @@ SSLKey::SSLKey(int keySize_, int ivLength_) {
   // most likely fails unless we're running as root, or a user-page-lock
   // kernel patch is applied..
   mlock(buffer, keySize + ivLength);
+
+  block_enc = EVP_CIPHER_CTX_new();
+  EVP_CIPHER_CTX_init(block_enc);
+  block_dec = EVP_CIPHER_CTX_new();
+  EVP_CIPHER_CTX_init(block_dec);
+  stream_enc = EVP_CIPHER_CTX_new();
+  EVP_CIPHER_CTX_init(stream_enc);
+  stream_dec = EVP_CIPHER_CTX_new();
+  EVP_CIPHER_CTX_init(stream_dec);
+  mac_ctx = HMAC_CTX_new();
+  HMAC_CTX_reset(mac_ctx);
 }
 
 SSLKey::~SSLKey() {
@@ -266,59 +274,50 @@ SSLKey::~SSLKey() {
   ivLength = 0;
   buffer = 0;
 
-  EVP_CIPHER_CTX_cleanup(&block_enc);
-  EVP_CIPHER_CTX_cleanup(&block_dec);
-  EVP_CIPHER_CTX_cleanup(&stream_enc);
-  EVP_CIPHER_CTX_cleanup(&stream_dec);
-
-  HMAC_CTX_cleanup(&mac_ctx);
+  EVP_CIPHER_CTX_free(block_enc);
+  EVP_CIPHER_CTX_free(block_dec);
+  EVP_CIPHER_CTX_free(stream_enc);
+  EVP_CIPHER_CTX_free(stream_dec);
+  HMAC_CTX_free(mac_ctx);
 
   pthread_mutex_destroy(&mutex);
 }
 
-inline unsigned char *KeyData(const shared_ptr<SSLKey> &key) {
+inline unsigned char *KeyData(const std::shared_ptr<SSLKey> &key) {
   return key->buffer;
 }
-inline unsigned char *IVData(const shared_ptr<SSLKey> &key) {
+inline unsigned char *IVData(const std::shared_ptr<SSLKey> &key) {
   return key->buffer + key->keySize;
 }
 
-void initKey(const shared_ptr<SSLKey> &key, const EVP_CIPHER *_blockCipher,
+void initKey(const std::shared_ptr<SSLKey> &key, const EVP_CIPHER *_blockCipher,
              const EVP_CIPHER *_streamCipher, int _keySize) {
   Lock lock(key->mutex);
   // initialize the cipher context once so that we don't have to do it for
   // every block..
-  EVP_CIPHER_CTX_init(&key->block_enc);
-  EVP_CIPHER_CTX_init(&key->block_dec);
-  EVP_CIPHER_CTX_init(&key->stream_enc);
-  EVP_CIPHER_CTX_init(&key->stream_dec);
+  EVP_EncryptInit_ex(key->block_enc, _blockCipher, NULL, NULL, NULL);
+  EVP_DecryptInit_ex(key->block_dec, _blockCipher, NULL, NULL, NULL);
+  EVP_EncryptInit_ex(key->stream_enc, _streamCipher, NULL, NULL, NULL);
+  EVP_DecryptInit_ex(key->stream_dec, _streamCipher, NULL, NULL, NULL);
 
-  EVP_EncryptInit_ex(&key->block_enc, _blockCipher, NULL, NULL, NULL);
-  EVP_DecryptInit_ex(&key->block_dec, _blockCipher, NULL, NULL, NULL);
-  EVP_EncryptInit_ex(&key->stream_enc, _streamCipher, NULL, NULL, NULL);
-  EVP_DecryptInit_ex(&key->stream_dec, _streamCipher, NULL, NULL, NULL);
+  EVP_CIPHER_CTX_set_key_length(key->block_enc, _keySize);
+  EVP_CIPHER_CTX_set_key_length(key->block_dec, _keySize);
+  EVP_CIPHER_CTX_set_key_length(key->stream_enc, _keySize);
+  EVP_CIPHER_CTX_set_key_length(key->stream_dec, _keySize);
 
-  EVP_CIPHER_CTX_set_key_length(&key->block_enc, _keySize);
-  EVP_CIPHER_CTX_set_key_length(&key->block_dec, _keySize);
-  EVP_CIPHER_CTX_set_key_length(&key->stream_enc, _keySize);
-  EVP_CIPHER_CTX_set_key_length(&key->stream_dec, _keySize);
+  EVP_CIPHER_CTX_set_padding(key->block_enc, 0);
+  EVP_CIPHER_CTX_set_padding(key->block_dec, 0);
+  EVP_CIPHER_CTX_set_padding(key->stream_enc, 0);
+  EVP_CIPHER_CTX_set_padding(key->stream_dec, 0);
 
-  EVP_CIPHER_CTX_set_padding(&key->block_enc, 0);
-  EVP_CIPHER_CTX_set_padding(&key->block_dec, 0);
-  EVP_CIPHER_CTX_set_padding(&key->stream_enc, 0);
-  EVP_CIPHER_CTX_set_padding(&key->stream_dec, 0);
+  EVP_EncryptInit_ex(key->block_enc, NULL, NULL, KeyData(key), NULL);
+  EVP_DecryptInit_ex(key->block_dec, NULL, NULL, KeyData(key), NULL);
+  EVP_EncryptInit_ex(key->stream_enc, NULL, NULL, KeyData(key), NULL);
+  EVP_DecryptInit_ex(key->stream_dec, NULL, NULL, KeyData(key), NULL);
 
-  EVP_EncryptInit_ex(&key->block_enc, NULL, NULL, KeyData(key), NULL);
-  EVP_DecryptInit_ex(&key->block_dec, NULL, NULL, KeyData(key), NULL);
-  EVP_EncryptInit_ex(&key->stream_enc, NULL, NULL, KeyData(key), NULL);
-  EVP_DecryptInit_ex(&key->stream_dec, NULL, NULL, KeyData(key), NULL);
-
-  HMAC_CTX_init(&key->mac_ctx);
-  HMAC_Init_ex(&key->mac_ctx, KeyData(key), _keySize, EVP_sha1(), 0);
+  HMAC_Init_ex(key->mac_ctx, KeyData(key), _keySize, EVP_sha1(), 0);
 }
 
-static RLogChannel *CipherInfo = DEF_CHANNEL("info/cipher", Log_Info);
-
 SSL_Cipher::SSL_Cipher(const Interface &iface_, const Interface &realIface_,
                        const EVP_CIPHER *blockCipher,
                        const EVP_CIPHER *streamCipher, int keySize_) {
@@ -331,16 +330,15 @@ SSL_Cipher::SSL_Cipher(const Interface &iface_, const Interface &realIface_,
 
   rAssert(_ivLength == 8 || _ivLength == 16);
 
-  rLog(CipherInfo, "allocated cipher %s, keySize %i, ivlength %i",
-       iface.name().c_str(), _keySize, _ivLength);
+  VLOG(1) << "allocated cipher " << iface.name() << ", keySize " << _keySize
+          << ", ivlength " << _ivLength;
 
   if ((EVP_CIPHER_key_length(_blockCipher) != (int)_keySize) &&
       iface.current() == 1) {
-    rWarning(
-        "Running in backward compatibilty mode for 1.0 - \n"
-        "key is really %i bits, not %i.\n"
-        "Consider creating a new filesystem and moving your data.",
-        EVP_CIPHER_key_length(_blockCipher) * 8, _keySize * 8);
+    RLOG(WARNING) << "Running in backward compatibilty mode for 1.0 - "
+                     "key is really "
+                  << EVP_CIPHER_key_length(_blockCipher) * 8 << " bits, not "
+                  << _keySize * 8;
   }
 }
 
@@ -358,7 +356,7 @@ Interface SSL_Cipher::interface() const { return realIface; }
 CipherKey SSL_Cipher::newKey(const char *password, int passwdLength,
                              int &iterationCount, long desiredDuration,
                              const unsigned char *salt, int saltLen) {
-  shared_ptr<SSLKey> key(new SSLKey(_keySize, _ivLength));
+  std::shared_ptr<SSLKey> key(new SSLKey(_keySize, _ivLength));
 
   if (iterationCount == 0) {
     // timed run, fills in iteration count
@@ -366,7 +364,7 @@ CipherKey SSL_Cipher::newKey(const char *password, int passwdLength,
         TimedPBKDF2(password, passwdLength, salt, saltLen, _keySize + _ivLength,
                     KeyData(key), 1000 * desiredDuration);
     if (res <= 0) {
-      rWarning("openssl error, PBKDF2 failed");
+      RLOG(WARNING) << "openssl error, PBKDF2 failed";
       return CipherKey();
     } else
       iterationCount = res;
@@ -375,7 +373,7 @@ CipherKey SSL_Cipher::newKey(const char *password, int passwdLength,
     if (PKCS5_PBKDF2_HMAC_SHA1(
             password, passwdLength, const_cast<unsigned char *>(salt), saltLen,
             iterationCount, _keySize + _ivLength, KeyData(key)) != 1) {
-      rWarning("openssl error, PBKDF2 failed");
+      RLOG(WARNING) << "openssl error, PBKDF2 failed";
       return CipherKey();
     }
   }
@@ -386,7 +384,7 @@ CipherKey SSL_Cipher::newKey(const char *password, int passwdLength,
 }
 
 CipherKey SSL_Cipher::newKey(const char *password, int passwdLength) {
-  shared_ptr<SSLKey> key(new SSLKey(_keySize, _ivLength));
+  std::shared_ptr<SSLKey> key(new SSLKey(_keySize, _ivLength));
 
   int bytes = 0;
   if (iface.current() > 1) {
@@ -398,8 +396,8 @@ CipherKey SSL_Cipher::newKey(const char *password, int passwdLength) {
 
     // the reason for moving from EVP_BytesToKey to BytesToKey function..
     if (bytes != (int)_keySize) {
-      rWarning("newKey: BytesToKey returned %i, expecting %i key bytes", bytes,
-               _keySize);
+      RLOG(WARNING) << "newKey: BytesToKey returned " << bytes << ", expecting "
+                    << _keySize << " key bytes";
     }
   } else {
     // for backward compatibility with filesystems created with 1:0
@@ -430,13 +428,13 @@ CipherKey SSL_Cipher::newRandomKey() {
   if (!randomize(tmpBuf, bufLen, true) || !randomize(saltBuf, saltLen, true))
     return CipherKey();
 
-  shared_ptr<SSLKey> key(new SSLKey(_keySize, _ivLength));
+  std::shared_ptr<SSLKey> key(new SSLKey(_keySize, _ivLength));
 
   // doesn't need to be versioned, because a random key is a random key..
   // Doesn't need to be reproducable..
   if (PKCS5_PBKDF2_HMAC_SHA1((char *)tmpBuf, bufLen, saltBuf, saltLen, 1000,
                              _keySize + _ivLength, KeyData(key)) != 1) {
-    rWarning("openssl error, PBKDF2 failed");
+    RLOG(WARNING) << "openssl error, PBKDF2 failed";
     return CipherKey();
   }
 
@@ -458,8 +456,8 @@ static uint64_t _checksum_64(SSLKey *key, const unsigned char *data,
   unsigned char md[EVP_MAX_MD_SIZE];
   unsigned int mdLen = EVP_MAX_MD_SIZE;
 
-  HMAC_Init_ex(&key->mac_ctx, 0, 0, 0, 0);
-  HMAC_Update(&key->mac_ctx, data, dataLen);
+  HMAC_Init_ex(key->mac_ctx, 0, 0, 0, 0);
+  HMAC_Update(key->mac_ctx, data, dataLen);
   if (chainedIV) {
     // toss in the chained IV as well
     uint64_t tmp = *chainedIV;
@@ -469,10 +467,10 @@ static uint64_t _checksum_64(SSLKey *key, const unsigned char *data,
       tmp >>= 8;
     }
 
-    HMAC_Update(&key->mac_ctx, h, 8);
+    HMAC_Update(key->mac_ctx, h, 8);
   }
 
-  HMAC_Final(&key->mac_ctx, md, &mdLen);
+  HMAC_Final(key->mac_ctx, md, &mdLen);
 
   rAssert(mdLen >= 8);
 
@@ -490,32 +488,31 @@ static uint64_t _checksum_64(SSLKey *key, const unsigned char *data,
 /**
  * Write "len" bytes of random data into "buf"
  *
- * See "man 3 RAND_bytes" for the effect of strongRandom
+ * We ignore the @strongRandom parameter because OpenSSL
+ * does not * offer a "weak" random generator
  */
 bool SSL_Cipher::randomize(unsigned char *buf, int len,
-                           bool strongRandom) const {
+                           bool /*strongRandom*/) const {
   // to avoid warnings of uninitialized data from valgrind
   memset(buf, 0, len);
-  int result;
-  if (strongRandom)
-    result = RAND_bytes(buf, len);
-  else
-    result = RAND_pseudo_bytes(buf, len);
 
+  int result = RAND_bytes(buf, len);
   if (result != 1) {
     char errStr[120];  // specs require string at least 120 bytes long..
     unsigned long errVal = 0;
-    if ((errVal = ERR_get_error()) != 0)
-      rWarning("openssl error: %s", ERR_error_string(errVal, errStr));
+    if ((errVal = ERR_get_error()) != 0) {
+      RLOG(WARNING) << "openssl error: " << ERR_error_string(errVal, errStr);
+    }
 
     return false;
-  } else
+  } else {
     return true;
+  }
 }
 
 uint64_t SSL_Cipher::MAC_64(const unsigned char *data, int len,
                             const CipherKey &key, uint64_t *chainedIV) const {
-  shared_ptr<SSLKey> mk = dynamic_pointer_cast<SSLKey>(key);
+  std::shared_ptr<SSLKey> mk = dynamic_pointer_cast<SSLKey>(key);
   uint64_t tmp = _checksum_64(mk.get(), data, len, chainedIV);
 
   if (chainedIV) *chainedIV = tmp;
@@ -525,7 +522,7 @@ uint64_t SSL_Cipher::MAC_64(const unsigned char *data, int len,
 
 CipherKey SSL_Cipher::readKey(const unsigned char *data,
                               const CipherKey &masterKey, bool checkKey) {
-  shared_ptr<SSLKey> mk = dynamic_pointer_cast<SSLKey>(masterKey);
+  std::shared_ptr<SSLKey> mk = dynamic_pointer_cast<SSLKey>(masterKey);
   rAssert(mk->keySize == _keySize);
 
   unsigned char tmpBuf[MAX_KEYLENGTH + MAX_IVLENGTH];
@@ -541,13 +538,14 @@ CipherKey SSL_Cipher::readKey(const unsigned char *data,
   // check for success
   unsigned int checksum2 = MAC_32(tmpBuf, _keySize + _ivLength, masterKey);
   if (checksum2 != checksum && checkKey) {
-    rDebug("checksum mismatch: expected %u, got %u", checksum, checksum2);
-    rDebug("on decode of %i bytes", _keySize + _ivLength);
+    VLOG(1) << "checksum mismatch: expected " << checksum << ", got "
+            << checksum2;
+    VLOG(1) << "on decode of " << _keySize + _ivLength << " bytes";
     memset(tmpBuf, 0, sizeof(tmpBuf));
     return CipherKey();
   }
 
-  shared_ptr<SSLKey> key(new SSLKey(_keySize, _ivLength));
+  std::shared_ptr<SSLKey> key(new SSLKey(_keySize, _ivLength));
 
   memcpy(key->buffer, tmpBuf, _keySize + _ivLength);
   memset(tmpBuf, 0, sizeof(tmpBuf));
@@ -559,11 +557,11 @@ CipherKey SSL_Cipher::readKey(const unsigned char *data,
 
 void SSL_Cipher::writeKey(const CipherKey &ckey, unsigned char *data,
                           const CipherKey &masterKey) {
-  shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
+  std::shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
   rAssert(key->keySize == _keySize);
   rAssert(key->ivLength == _ivLength);
 
-  shared_ptr<SSLKey> mk = dynamic_pointer_cast<SSLKey>(masterKey);
+  std::shared_ptr<SSLKey> mk = dynamic_pointer_cast<SSLKey>(masterKey);
   rAssert(mk->keySize == _keySize);
   rAssert(mk->ivLength == _ivLength);
 
@@ -587,8 +585,8 @@ void SSL_Cipher::writeKey(const CipherKey &ckey, unsigned char *data,
 }
 
 bool SSL_Cipher::compareKey(const CipherKey &A, const CipherKey &B) const {
-  shared_ptr<SSLKey> key1 = dynamic_pointer_cast<SSLKey>(A);
-  shared_ptr<SSLKey> key2 = dynamic_pointer_cast<SSLKey>(B);
+  std::shared_ptr<SSLKey> key1 = dynamic_pointer_cast<SSLKey>(A);
+  std::shared_ptr<SSLKey> key2 = dynamic_pointer_cast<SSLKey>(B);
 
   rAssert(key1->keySize == _keySize);
   rAssert(key2->keySize == _keySize);
@@ -626,7 +624,7 @@ int SSL_Cipher::cipherBlockSize() const {
  * requirement for "seed" is that is must be unique.
  */
 void SSL_Cipher::setIVec(unsigned char *ivec, uint64_t seed,
-                         const shared_ptr<SSLKey> &key) const {
+                         const std::shared_ptr<SSLKey> &key) const {
   if (iface.current() >= 3) {
     memcpy(ivec, IVData(key), _ivLength);
 
@@ -639,10 +637,10 @@ void SSL_Cipher::setIVec(unsigned char *ivec, uint64_t seed,
     }
 
     // combine ivec and seed with HMAC
-    HMAC_Init_ex(&key->mac_ctx, 0, 0, 0, 0);
-    HMAC_Update(&key->mac_ctx, ivec, _ivLength);
-    HMAC_Update(&key->mac_ctx, md, 8);
-    HMAC_Final(&key->mac_ctx, md, &mdLen);
+    HMAC_Init_ex(key->mac_ctx, 0, 0, 0, 0);
+    HMAC_Update(key->mac_ctx, ivec, _ivLength);
+    HMAC_Update(key->mac_ctx, md, 8);
+    HMAC_Final(key->mac_ctx, md, &mdLen);
     rAssert(mdLen >= _ivLength);
 
     memcpy(ivec, md, _ivLength);
@@ -658,7 +656,7 @@ void SSL_Cipher::setIVec(unsigned char *ivec, uint64_t seed,
     decrypting the file).
   */
 void SSL_Cipher::setIVec_old(unsigned char *ivec, unsigned int seed,
-                             const shared_ptr<SSLKey> &key) const {
+                             const std::shared_ptr<SSLKey> &key) const {
   /* These multiplication constants chosen as they represent (non optimal)
      Golumb rulers, the idea being to spread around the information in the
      seed.
@@ -674,21 +672,21 @@ void SSL_Cipher::setIVec_old(unsigned char *ivec, unsigned int seed,
   ivec[0] ^= (var1 >> 24) & 0xff;
   ivec[1] ^= (var2 >> 16) & 0xff;
   ivec[2] ^= (var1 >> 8) & 0xff;
-  ivec[3] ^= (var2) & 0xff;
+  ivec[3] ^= (var2)&0xff;
   ivec[4] ^= (var2 >> 24) & 0xff;
   ivec[5] ^= (var1 >> 16) & 0xff;
   ivec[6] ^= (var2 >> 8) & 0xff;
-  ivec[7] ^= (var1) & 0xff;
+  ivec[7] ^= (var1)&0xff;
 
   if (_ivLength > 8) {
-    ivec[8 + 0] ^= (var1) & 0xff;
+    ivec[8 + 0] ^= (var1)&0xff;
     ivec[8 + 1] ^= (var2 >> 8) & 0xff;
     ivec[8 + 2] ^= (var1 >> 16) & 0xff;
     ivec[8 + 3] ^= (var2 >> 24) & 0xff;
     ivec[8 + 4] ^= (var1 >> 24) & 0xff;
     ivec[8 + 5] ^= (var2 >> 16) & 0xff;
     ivec[8 + 6] ^= (var1 >> 8) & 0xff;
-    ivec[8 + 7] ^= (var2) & 0xff;
+    ivec[8 + 7] ^= (var2)&0xff;
   }
 }
 
@@ -722,7 +720,7 @@ static void unshuffleBytes(unsigned char *buf, int size) {
 bool SSL_Cipher::streamEncode(unsigned char *buf, int size, uint64_t iv64,
                               const CipherKey &ckey) const {
   rAssert(size > 0);
-  shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
+  std::shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
   rAssert(key->keySize == _keySize);
   rAssert(key->ivLength == _ivLength);
 
@@ -734,22 +732,22 @@ bool SSL_Cipher::streamEncode(unsigned char *buf, int size, uint64_t iv64,
   shuffleBytes(buf, size);
 
   setIVec(ivec, iv64, key);
-  EVP_EncryptInit_ex(&key->stream_enc, NULL, NULL, NULL, ivec);
-  EVP_EncryptUpdate(&key->stream_enc, buf, &dstLen, buf, size);
-  EVP_EncryptFinal_ex(&key->stream_enc, buf + dstLen, &tmpLen);
+  EVP_EncryptInit_ex(key->stream_enc, NULL, NULL, NULL, ivec);
+  EVP_EncryptUpdate(key->stream_enc, buf, &dstLen, buf, size);
+  EVP_EncryptFinal_ex(key->stream_enc, buf + dstLen, &tmpLen);
 
   flipBytes(buf, size);
   shuffleBytes(buf, size);
 
   setIVec(ivec, iv64 + 1, key);
-  EVP_EncryptInit_ex(&key->stream_enc, NULL, NULL, NULL, ivec);
-  EVP_EncryptUpdate(&key->stream_enc, buf, &dstLen, buf, size);
-  EVP_EncryptFinal_ex(&key->stream_enc, buf + dstLen, &tmpLen);
+  EVP_EncryptInit_ex(key->stream_enc, NULL, NULL, NULL, ivec);
+  EVP_EncryptUpdate(key->stream_enc, buf, &dstLen, buf, size);
+  EVP_EncryptFinal_ex(key->stream_enc, buf + dstLen, &tmpLen);
 
   dstLen += tmpLen;
   if (dstLen != size) {
-    rError("encoding %i bytes, got back %i (%i in final_ex)", size, dstLen,
-           tmpLen);
+    RLOG(ERROR) << "encoding " << size << " bytes, got back " << dstLen << " ("
+                << tmpLen << " in final_ex)";
   }
 
   return true;
@@ -758,7 +756,7 @@ bool SSL_Cipher::streamEncode(unsigned char *buf, int size, uint64_t iv64,
 bool SSL_Cipher::streamDecode(unsigned char *buf, int size, uint64_t iv64,
                               const CipherKey &ckey) const {
   rAssert(size > 0);
-  shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
+  std::shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
   rAssert(key->keySize == _keySize);
   rAssert(key->ivLength == _ivLength);
 
@@ -768,24 +766,24 @@ bool SSL_Cipher::streamDecode(unsigned char *buf, int size, uint64_t iv64,
   int dstLen = 0, tmpLen = 0;
 
   setIVec(ivec, iv64 + 1, key);
-  EVP_DecryptInit_ex(&key->stream_dec, NULL, NULL, NULL, ivec);
-  EVP_DecryptUpdate(&key->stream_dec, buf, &dstLen, buf, size);
-  EVP_DecryptFinal_ex(&key->stream_dec, buf + dstLen, &tmpLen);
+  EVP_DecryptInit_ex(key->stream_dec, NULL, NULL, NULL, ivec);
+  EVP_DecryptUpdate(key->stream_dec, buf, &dstLen, buf, size);
+  EVP_DecryptFinal_ex(key->stream_dec, buf + dstLen, &tmpLen);
 
   unshuffleBytes(buf, size);
   flipBytes(buf, size);
 
   setIVec(ivec, iv64, key);
-  EVP_DecryptInit_ex(&key->stream_dec, NULL, NULL, NULL, ivec);
-  EVP_DecryptUpdate(&key->stream_dec, buf, &dstLen, buf, size);
-  EVP_DecryptFinal_ex(&key->stream_dec, buf + dstLen, &tmpLen);
+  EVP_DecryptInit_ex(key->stream_dec, NULL, NULL, NULL, ivec);
+  EVP_DecryptUpdate(key->stream_dec, buf, &dstLen, buf, size);
+  EVP_DecryptFinal_ex(key->stream_dec, buf + dstLen, &tmpLen);
 
   unshuffleBytes(buf, size);
 
   dstLen += tmpLen;
   if (dstLen != size) {
-    rError("encoding %i bytes, got back %i (%i in final_ex)", size, dstLen,
-           tmpLen);
+    RLOG(ERROR) << "decoding " << size << " bytes, got back " << dstLen << " ("
+                << tmpLen << " in final_ex)";
   }
 
   return true;
@@ -794,14 +792,14 @@ bool SSL_Cipher::streamDecode(unsigned char *buf, int size, uint64_t iv64,
 bool SSL_Cipher::blockEncode(unsigned char *buf, int size, uint64_t iv64,
                              const CipherKey &ckey) const {
   rAssert(size > 0);
-  shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
+  std::shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
   rAssert(key->keySize == _keySize);
   rAssert(key->ivLength == _ivLength);
 
   // data must be integer number of blocks
-  const int blockMod = size % EVP_CIPHER_CTX_block_size(&key->block_enc);
+  const int blockMod = size % EVP_CIPHER_CTX_block_size(key->block_enc);
   if (blockMod != 0)
-    throw ERROR("Invalid data size, not multiple of block size");
+    throw Error("Invalid data size, not multiple of block size");
 
   Lock lock(key->mutex);
 
@@ -810,14 +808,14 @@ bool SSL_Cipher::blockEncode(unsigned char *buf, int size, uint64_t iv64,
   int dstLen = 0, tmpLen = 0;
   setIVec(ivec, iv64, key);
 
-  EVP_EncryptInit_ex(&key->block_enc, NULL, NULL, NULL, ivec);
-  EVP_EncryptUpdate(&key->block_enc, buf, &dstLen, buf, size);
-  EVP_EncryptFinal_ex(&key->block_enc, buf + dstLen, &tmpLen);
+  EVP_EncryptInit_ex(key->block_enc, NULL, NULL, NULL, ivec);
+  EVP_EncryptUpdate(key->block_enc, buf, &dstLen, buf, size);
+  EVP_EncryptFinal_ex(key->block_enc, buf + dstLen, &tmpLen);
   dstLen += tmpLen;
 
   if (dstLen != size) {
-    rError("encoding %i bytes, got back %i (%i in final_ex)", size, dstLen,
-           tmpLen);
+    RLOG(ERROR) << "encoding " << size << " bytes, got back " << dstLen << " ("
+                << tmpLen << " in final_ex)";
   }
 
   return true;
@@ -826,14 +824,14 @@ bool SSL_Cipher::blockEncode(unsigned char *buf, int size, uint64_t iv64,
 bool SSL_Cipher::blockDecode(unsigned char *buf, int size, uint64_t iv64,
                              const CipherKey &ckey) const {
   rAssert(size > 0);
-  shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
+  std::shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
   rAssert(key->keySize == _keySize);
   rAssert(key->ivLength == _ivLength);
 
   // data must be integer number of blocks
-  const int blockMod = size % EVP_CIPHER_CTX_block_size(&key->block_dec);
+  const int blockMod = size % EVP_CIPHER_CTX_block_size(key->block_dec);
   if (blockMod != 0)
-    throw ERROR("Invalid data size, not multiple of block size");
+    throw Error("Invalid data size, not multiple of block size");
 
   Lock lock(key->mutex);
 
@@ -842,17 +840,19 @@ bool SSL_Cipher::blockDecode(unsigned char *buf, int size, uint64_t iv64,
   int dstLen = 0, tmpLen = 0;
   setIVec(ivec, iv64, key);
 
-  EVP_DecryptInit_ex(&key->block_dec, NULL, NULL, NULL, ivec);
-  EVP_DecryptUpdate(&key->block_dec, buf, &dstLen, buf, size);
-  EVP_DecryptFinal_ex(&key->block_dec, buf + dstLen, &tmpLen);
+  EVP_DecryptInit_ex(key->block_dec, NULL, NULL, NULL, ivec);
+  EVP_DecryptUpdate(key->block_dec, buf, &dstLen, buf, size);
+  EVP_DecryptFinal_ex(key->block_dec, buf + dstLen, &tmpLen);
   dstLen += tmpLen;
 
   if (dstLen != size) {
-    rError("decoding %i bytes, got back %i (%i in final_ex)", size, dstLen,
-           tmpLen);
+    RLOG(ERROR) << "decoding " << size << " bytes, got back " << dstLen << " ("
+                << tmpLen << " in final_ex)";
   }
 
   return true;
 }
 
 bool SSL_Cipher::Enabled() { return true; }
+
+}  // namespace encfs
