@@ -2,7 +2,7 @@
 
 # Test EncFS normal and paranoid mode
 
-use Test::More tests => 104;
+use Test::More tests => 116;
 use File::Path;
 use File::Copy;
 use File::Temp;
@@ -22,6 +22,7 @@ my $tempDir = $ENV{'TMPDIR'} || "/tmp";
 sub runTests
 {
     my $mode = shift;
+    print STDERR "\nrunTests: mode=$mode\n";
 
     &newWorkingDir;
 
@@ -47,7 +48,9 @@ sub runTests
     &internalModification;
     &grow;
     &umask0777;
+    &create_unmount_remount;
 
+    &configFromPipe;
     &cleanup;
 }
 
@@ -189,13 +192,13 @@ sub truncate
 sub fileCreation
 {
     # create a file
-    qx(df -ah > "$crypt/df.txt" 2> /dev/null);
-    ok( -f "$crypt/df.txt", "file created" );
+    qx(date > "$crypt/df.txt");
+    ok( -f "$crypt/df.txt", "file created" ) || BAIL_OUT("file create failed");
 
     # ensure there is an encrypted version.
     my $c = encName("df.txt");
     cmp_ok( length($c), '>', 8, "encrypted name ok" );
-    ok( -f "$raw/$c", "encrypted file created" );
+    ok( -f "$raw/$c", "encrypted file $raw/$c created" );
 
     # check contents
     my $count = qx(grep -c crypt-$$ "$crypt/df.txt");
@@ -305,11 +308,19 @@ sub mount
     mkdir($crypt)  || BAIL_OUT("Could not create $crypt: $!");
 
     delete $ENV{"ENCFS6_CONFIG"};
+    remount($args);
+    ok( $? == 0, "encfs command returns 0") || BAIL_OUT("");
+    ok( -f "$raw/.encfs6.xml",  "created control file") || BAIL_OUT("");
+}
+
+# Helper function
+# Mount without any prior checks
+sub remount
+{
+    my $args = shift;
     my $cmdline = "./build/encfs --extpass=\"echo test\" $args $raw $crypt 2>&1";
     #                                  This makes sure we get to see stderr ^
-    my $status = system($cmdline);
-    ok( $status == 0, "encfs command returns 0") || BAIL_OUT("");
-    ok( -f "$raw/.encfs6.xml",  "created control file") || BAIL_OUT("");
+    system($cmdline);
 }
 
 # Helper function
@@ -334,3 +345,50 @@ sub umask0777
     close($fh);
     umask($old);
 }
+
+# Test that we can read the configuration from a named pipe
+# Regression test for https://github.com/vgough/encfs/issues/253
+sub configFromPipe
+{
+    portable_unmount($crypt);
+    rename("$raw/.encfs6.xml", "$raw/.encfs6.xml.orig");
+    system("mkfifo $raw/.encfs6.xml");
+    my $child = fork();
+    unless ($child) {
+        &remount("--standard");
+        exit;
+    }
+    system("cat $raw/.encfs6.xml.orig > $raw/.encfs6.xml");
+    waitpid($child, 0);
+    ok( 0 == $?, "encfs mount with named pipe based config failed");
+}
+
+sub create_unmount_remount
+{
+    my $crypt = "$workingDir/create_remount.crypt";
+    my $mnt = "$workingDir/create_remount.mnt";
+    mkdir($crypt) || BAIL_OUT($!);
+    mkdir($mnt)  || BAIL_OUT($!);
+
+    system("./build/encfs --standard --extpass=\"echo test\" $crypt $mnt 2>&1");
+    ok( $? == 0, "encfs command returns 0") || return;
+    ok( -f "$crypt/.encfs6.xml",  "created control file") || return;
+
+    # Write some text
+    my $contents = "hello world\n";
+    ok( open(OUT, "> $mnt/test_file_1"), "write content");
+    print OUT $contents;
+    close OUT;
+
+    # Unmount
+    portable_unmount($mnt);
+
+    # Mount again
+    system("./build/encfs --extpass=\"echo test\" $crypt $mnt 2>&1");
+    ok( $? == 0, "encfs command returns 0") || return;
+
+    # Check if content is still there
+    checkContents("$mnt/test_file_1", $contents);
+
+    portable_unmount($mnt);
+}
